/* Index access routines */ 
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "utils.h"

#include "ihipva.h" 

int load_idx_hip_va (char* ifile,idx_hip_va* index) 
{
/* Routine to load an index from a file. The structure for the index must
 * already exist as must the file of course
 */
   idx_hip_va idx;
   FILE *fp;
   int i;

   if ( (fp=fopen(ifile,"r"))==NULL)
   {
	fprintf(stderr,"load_idx_hip_va unable to open %s \n",ifile);
	return (-1);
   }
   if (index == NULL) 
   {
      fprintf (stderr,"load_idx_hip_va NULL pointer for index\n");
      return (-1);
   }
   /*printf("Loading index idx_hip_va from %s.. ",ifile);*/
   for (i=0;i<idx_hip_va_entries;i++)
   {
	if ( (getnext_idx_hip_va (fp,&index[i])) != 0)
   	{
	   fprintf(stderr,"load_idx_hip_va problem with entry %d \n",i);
	   break;
	}
   /*     if  ( (i % 5000) == 0) printf("#");*/
   }
   if ( i < idx_hip_va_entries) 
   {
	fprintf(stderr,"Premature end of index file \n");
   	return -1;
   }
   /* printf(" done \n"); */
   return 0;
} /* End of load_idx_hip_va*/

int getnext_idx_hip_va (FILE* fp, idx_hip_va* entry)
{
   char buffer[idx_hip_va_REC_LEN + 2];
   if (fgets((char *)&buffer,idx_hip_va_REC_LEN+1,fp))
   { 
	char delimiter[3]="|\r\0";
	char *token=strtok((char *)&buffer,(char *)&delimiter); 
	strAsINT(token,&entry->HIPID);
	token=strtok(NULL,delimiter);
	strAsINT(token,&entry->ANNEX);
	token=strtok(NULL,delimiter);
	strAsINT(token,&entry->RECNO);
   return 0;
   }
   else
        return -1;
} /* End of getnext_idx_hip_va */

idx_hip_va* find_idx_hip_va (INT* key)
{
   static int init=0;
   static idx_hip_va index[idx_hip_va_entries];
   int mid,lpos,rpos,found;
   if (init == 0)
   {
        if (load_idx_hip_va("hip_va.idx",(idx_hip_va*)&index) !=0)
             return NULL;
        init=1;
   }

   /*  Binary search */ 
   rpos = key->value; 
   lpos = 0;
   if (rpos > idx_hip_va_entries) rpos = idx_hip_va_entries;
   found = 0;
   while (!found && !(lpos==mid && rpos==mid || lpos > rpos))
   {
	mid =  (rpos + lpos) /2;
        found = index[mid].HIPID.value == key->value;
     	if (key->value > index[mid].HIPID.value )
	   lpos = mid+1;
	else
	   if (key->value < index[mid].HIPID.value)
	      rpos = mid;
	/*printf (" %d,%d Betweeen %d,%d and %d,%d Key = %d\n",mid,index[mid].HIPID.value,lpos,index[lpos].HIPID.value,rpos,index[rpos].HIPID.value,key->value);*/
   }
   if (found)
	return &index[mid];
   else
   {
      return NULL;
   }
   
} /* End of find_idx_hip_va */


